@@ -17,7 +17,7 @@ GameInfoFlags = [
 	"BUG_DDRACE_INPUT", "BUG_FNG_LASER_RANGE", "BUG_VANILLA_BOUNCE",
 	"PREDICT_FNG", "PREDICT_DDRACE", "PREDICT_DDRACE_TILES", "PREDICT_VANILLA",
 	"ENTITIES_DDNET", "ENTITIES_DDRACE", "ENTITIES_RACE", "ENTITIES_FNG",
-	"ENTITIES_VANILLA", "DONT_MASK_ENTITIES",
+	"ENTITIES_VANILLA", "DONT_MASK_ENTITIES", "ENTITIES_BW",
 ]
 ExPlayerFlags = ["AFK", "PAUSED", "SPEC"]
 
@@ -52,7 +52,7 @@ enum
 
 enum
 {
-	GAMEINFO_CURVERSION=3,
+	GAMEINFO_CURVERSION=4,
 };
 '''
 
@@ -269,6 +269,11 @@ Objects = [
 	NetObjectEx("MyOwnEvent", "my-own-event@heinrich5991.de", [
 		NetIntAny("m_Test"),
 	]),
+
+	NetObjectEx("SpecChar", "spec-char@netobj.ddnet.tw", [
+		NetIntAny("m_X"),
+		NetIntAny("m_Y"),
+	]),
 ]
 
 Messages = [
@@ -408,14 +413,12 @@ Messages = [
 		NetIntAny("m_PlayerTimeBest"),
 	]),
 
-	NetMessage("Sv_PlayerTime", [
-		NetIntAny("m_Time"),
-		NetIntRange("m_ClientID", 0, 'MAX_CLIENTS-1'),
-	]),
+	NetMessage("Unused", []),
 
 	NetMessage("Sv_TeamsState", []),
 
-	NetMessage("Cl_ShowOthers", [
+	# deprecated, use showothers@netmsg.ddnet.tw instead
+	NetMessage("Cl_ShowOthersLegacy", [
 		NetBool("m_Show"),
 	]),
 # Can't add any NetMessages here!
@@ -423,4 +426,13 @@ Messages = [
 	NetMessageEx("Sv_MyOwnMessage", "my-own-message@heinrich5991.de", [
 		NetIntAny("m_Test"),
 	]),
+
+	NetMessageEx("Cl_ShowDistance", "show-distance@netmsg.ddnet.tw", [
+		NetIntAny("m_X"),
+		NetIntAny("m_Y"),
+	]),
+
+	NetMessageEx("Cl_ShowOthers", "showothers@netmsg.ddnet.tw", [
+		NetIntRange("m_Show", 0, 2),
+	]),
 ]
