@@ -103,7 +103,7 @@ void dbg_assert_imp(const char *filename, int line, int test, const char *msg)
 	}
 }
 
-void dbg_break_imp()
+void dbg_break_imp(void)
 {
 #ifdef __GNUC__
 	__builtin_trap();
@@ -231,7 +231,7 @@ void dbg_logger(DBG_LOGGER logger, DBG_LOGGER_FINISH finish, void *user)
 	num_loggers++;
 }
 
-void dbg_logger_stdout()
+void dbg_logger_stdout(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	dbg_logger(logger_stdout_sync, 0, 0);
@@ -240,7 +240,7 @@ void dbg_logger_stdout()
 #endif
 }
 
-void dbg_logger_debugger()
+void dbg_logger_debugger(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	dbg_logger(logger_debugger, 0, 0);
@@ -735,7 +735,7 @@ void thread_wait(void *thread)
 #endif
 }
 
-void thread_yield()
+void thread_yield(void)
 {
 #if defined(CONF_FAMILY_UNIX)
 	int result = sched_yield();
@@ -793,7 +793,7 @@ typedef CRITICAL_SECTION LOCKINTERNAL;
 	#error not implemented on this platform
 #endif
 
-LOCK lock_create()
+LOCK lock_create(void)
 {
 	LOCKINTERNAL *lock = (LOCKINTERNAL *)malloc(sizeof(*lock));
 #if defined(CONF_FAMILY_UNIX)
@@ -917,13 +917,13 @@ void sphore_destroy(SEMAPHORE *sem)
 
 static int new_tick = -1;
 
-void set_new_tick()
+void set_new_tick(void)
 {
 	new_tick = 1;
 }
 
 /* -----  time ----- */
-int64 time_get_impl()
+int64 time_get_impl(void)
 {
 	static int64 last = 0;
 	{
@@ -964,7 +964,7 @@ int64 time_get_impl()
 	}
 }
 
-int64 time_get()
+int64 time_get(void)
 {
 	static int64 last = 0;
 	if(new_tick == 0)
@@ -976,7 +976,7 @@ int64 time_get()
 	return last;
 }
 
-int64 time_freq()
+int64 time_freq(void)
 {
 #if defined(CONF_PLATFORM_MACOSX)
 	return 1000000000;
@@ -991,7 +991,7 @@ int64 time_freq()
 #endif
 }
 
-int64 time_get_microseconds()
+int64 time_get_microseconds(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	return (time_get_impl() * (int64)1000000) / time_freq();
@@ -1902,7 +1902,7 @@ int net_tcp_close(NETSOCKET sock)
 	return priv_net_close_all_sockets(sock);
 }
 
-int net_errno()
+int net_errno(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	return WSAGetLastError();
@@ -1911,7 +1911,7 @@ int net_errno()
 #endif
 }
 
-int net_would_block()
+int net_would_block(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	return net_errno() == WSAEWOULDBLOCK;
@@ -1920,7 +1920,7 @@ int net_would_block()
 #endif
 }
 
-int net_init()
+int net_init(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	WSADATA wsaData;
@@ -1933,7 +1933,7 @@ int net_init()
 }
 
 #if defined(CONF_FAMILY_UNIX)
-UNIXSOCKET net_unix_create_unnamed()
+UNIXSOCKET net_unix_create_unnamed(void)
 {
 	return socket(AF_UNIX, SOCK_DGRAM, 0);
 }
@@ -2292,7 +2292,7 @@ int net_socket_read_wait(NETSOCKET sock, int time)
 	return 0;
 }
 
-int time_timestamp()
+int time_timestamp(void)
 {
 	return time(0);
 }
@@ -3223,7 +3223,7 @@ const char *str_next_token(const char *str, const char *delim, char *buffer, int
 	return tok + len;
 }
 
-int pid()
+int pid(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	return _getpid();
@@ -3247,7 +3247,22 @@ void shell_execute(const char *file)
 #endif
 }
 
-int os_is_winxp_or_lower(unsigned int major, unsigned int minor)
+int open_link(const char *link)
+{
+	char aBuf[512];
+#if defined(CONF_FAMILY_WINDOWS)
+	str_format(aBuf, sizeof(aBuf), "start %s", link);
+	return (uintptr_t)ShellExecuteA(NULL, "open", link, NULL, NULL, SW_SHOWDEFAULT) > 32;
+#elif defined(CONF_PLATFORM_LINUX)
+	str_format(aBuf, sizeof(aBuf), "xdg-open %s >/dev/null 2>&1 &", link);
+	return system(aBuf) == 0;
+#elif defined(CONF_FAMILY_UNIX)
+	str_format(aBuf, sizeof(aBuf), "open %s &", link);
+	return system(aBuf) == 0;
+#endif
+}
+
+int os_is_winxp_or_lower(void)
 {
 #if defined(CONF_FAMILY_WINDOWS)
 	static const DWORD WINXP_MAJOR = 5;
@@ -3275,7 +3290,7 @@ struct SECURE_RANDOM_DATA
 
 static struct SECURE_RANDOM_DATA secure_random_data = { 0 };
 
-int secure_random_init()
+int secure_random_init(void)
 {
 	if(secure_random_data.initialized)
 	{
@@ -3363,7 +3378,7 @@ void secure_random_fill(void *bytes, unsigned length)
 #endif
 }
 
-int secure_rand()
+int secure_rand(void)
 {
 	unsigned int i;
 	secure_random_fill(&i, sizeof(i));
