@@ -103,6 +103,8 @@ class CClient : public IClient, public CDemoPlayer::IListener
 
 	char m_aServerAddressStr[256];
 
+	CUuid m_ConnectionID;
+
 	unsigned m_SnapshotParts[2];
 	int64 m_LocalStartTime;
 
@@ -274,8 +276,8 @@ public:
 	void SendInput();
 
 	// TODO: OPT: do this a lot smarter!
-	virtual int *GetInput(int Tick);
-	virtual int *GetDirectInput(int Tick);
+	virtual int *GetInput(int Tick, int IsDummy);
+	virtual int *GetDirectInput(int Tick, int IsDummy);
 
 	const char *LatestVersion();
 
@@ -428,6 +430,7 @@ public:
 	void ToggleWindowBordered();
 	void ToggleWindowVSync();
 	void LoadFont();
+	void Notify(const char *pTitle, const char *pMessage);
 
 	// DDRace
 
