@@ -561,10 +561,11 @@ class IGraphicsBackend
 public:
 	enum
 	{
-		INITFLAG_FULLSCREEN = 1,
-		INITFLAG_VSYNC = 2,
-		INITFLAG_RESIZABLE = 4,
-		INITFLAG_BORDERLESS = 8,
+		INITFLAG_FULLSCREEN = 1<<0,
+		INITFLAG_VSYNC = 1<<1,
+		INITFLAG_RESIZABLE = 1<<2,
+		INITFLAG_BORDERLESS = 1<<3,
+		INITFLAG_HIGHDPI = 1<<4,
 	};
 
 	virtual ~IGraphicsBackend() {}
