@@ -17,7 +17,7 @@ public:
 
 	bool Connect();
 	void Disconnect();
-	void CreateTables();
+	bool CreateTables();
 
 	void executeSql(const char *pCommand);
 	void executeSqlQuery(const char *pQuery);
@@ -30,6 +30,7 @@ public:
 	const char* GetPass() { return m_aPass; }
 	const char* GetIP() { return m_aIp; }
 	int GetPort() { return m_Port; }
+	sql::Connection *Connection() const { return m_pConnection; }
 
 	static int ms_NumReadServer;
 	static int ms_NumWriteServer;
