@@ -1,6 +1,8 @@
 #ifndef ENGINE_SERVER_SQL_STRING_HELPERS_H
 #define ENGINE_SERVER_SQL_STRING_HELPERS_H
 
+#include <base/system.h>
+
 namespace sqlstr
 {
 
@@ -30,7 +32,7 @@ public:
 	const char* Str() const { return m_aString; }
 	const char* ClrStr() const { return m_aClearString; }
 
-	CSqlString& operator = (const char *pStr)
+	CSqlString& operator=(const char *pStr)
 	{
 		str_copy(m_aString, pStr, size);
 		str_copy(m_aClearString, pStr, size);
@@ -38,6 +40,11 @@ public:
 		return *this;
 	}
 
+	bool operator<(const CSqlString& other) const
+	{
+		return str_comp(m_aString, other.m_aString) < 0;
+	}
+
 private:
 	char m_aString[size];
 	char m_aClearString[size * 2 - 1];
