@@ -10,6 +10,7 @@
 #include "network.h"
 #include <engine/message.h>
 #include <engine/shared/protocol.h>
+#include <game/generated/protocol.h>
 
 const int DummyMapCrc = 0x6c760ac4;
 unsigned char g_aDummyMapData[] = {
@@ -41,7 +42,6 @@ unsigned char g_aDummyMapData[] = {
 	0xc2, 0x00, 0x00, 0x38, 0x00, 0x05
 };
 
-
 static SECURITY_TOKEN ToSecurityToken(const unsigned char *pData)
 {
 	return (int)pData[0] | (pData[1] << 8) | (pData[2] << 16) | (pData[3] << 24);
@@ -147,7 +147,7 @@ SECURITY_TOKEN CNetServer::GetToken(const NETADDR &Addr)
 	SHA256_CTX Sha256;
 	sha256_init(&Sha256);
 	sha256_update(&Sha256, (unsigned char*)m_SecurityTokenSeed, sizeof(m_SecurityTokenSeed));
-	sha256_update(&Sha256, (unsigned char*)&Addr, sizeof(Addr));
+	sha256_update(&Sha256, (unsigned char*)&Addr, sizeof(20)); //omit port, bad idea?
 
 	SECURITY_TOKEN SecurityToken = ToSecurityToken(sha256_finish(&Sha256).data);
 
@@ -214,12 +214,19 @@ bool CNetServer::Connlimit(NETADDR Addr)
 	return false;
 }
 
-int CNetServer::TryAcceptClient(NETADDR &Addr, SECURITY_TOKEN SecurityToken, bool VanillaAuth)
+int CNetServer::TryAcceptClient(NETADDR &Addr, SECURITY_TOKEN SecurityToken, bool VanillaAuth, bool Sixup, SECURITY_TOKEN Token)
 {
+	if(Sixup && !g_Config.m_SvSixup)
+	{
+		const char Msg[] = "0.7 connections are not accepted at this time";
+		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CLOSE, Msg, sizeof(Msg), SecurityToken, Sixup);
+		return -1; // failed to add client?
+	}
+
 	if (Connlimit(Addr))
 	{
 		const char Msg[] = "Too many connections in a short time";
-		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CLOSE, Msg, sizeof(Msg), SecurityToken);
+		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CLOSE, Msg, sizeof(Msg), SecurityToken, Sixup);
 		return -1; // failed to add client
 	}
 
@@ -228,7 +235,7 @@ int CNetServer::TryAcceptClient(NETADDR &Addr, SECURITY_TOKEN SecurityToken, boo
 	{
 		char aBuf[128];
 		str_format(aBuf, sizeof(aBuf), "Only %d players with the same IP are allowed", m_MaxClientsPerIP);
-		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CLOSE, aBuf, str_length(aBuf) + 1, SecurityToken);
+		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CLOSE, aBuf, str_length(aBuf) + 1, SecurityToken, Sixup);
 		return -1; // failed to add client
 	}
 
@@ -245,13 +252,13 @@ int CNetServer::TryAcceptClient(NETADDR &Addr, SECURITY_TOKEN SecurityToken, boo
 	if (Slot == -1)
 	{
 		const char FullMsg[] = "This server is full";
-		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CLOSE, FullMsg, sizeof(FullMsg), SecurityToken);
+		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CLOSE, FullMsg, sizeof(FullMsg), SecurityToken, Sixup);
 
 		return -1; // failed to add client
 	}
 
 	// init connection slot
-	m_aSlots[Slot].m_Connection.DirectInit(Addr, SecurityToken);
+	m_aSlots[Slot].m_Connection.DirectInit(Addr, SecurityToken, Token, Sixup);
 
 	if (VanillaAuth)
 	{
@@ -273,7 +280,7 @@ int CNetServer::TryAcceptClient(NETADDR &Addr, SECURITY_TOKEN SecurityToken, boo
 	if (VanillaAuth)
 		m_pfnNewClientNoAuth(Slot, m_UserPtr);
 	else
-		m_pfnNewClient(Slot, m_UserPtr);
+		m_pfnNewClient(Slot, m_UserPtr, Sixup);
 
 	return Slot; // done
 }
@@ -546,6 +553,42 @@ void CNetServer::OnTokenCtrlMsg(NETADDR &Addr, int ControlMsg, const CNetPacketC
 	}
 }
 
+int CNetServer::OnSixupCtrlMsg(NETADDR &Addr, CNetChunk *pChunk, int ControlMsg, const CNetPacketConstruct &Packet, SECURITY_TOKEN &ResponseToken, SECURITY_TOKEN Token)
+{
+	if(m_RecvUnpacker.m_Data.m_DataSize < 5 || ClientExists(Addr))
+		return 0; // silently ignore
+
+	mem_copy(&ResponseToken, Packet.m_aChunkData+1, 4);
+
+	if(ControlMsg == 5)
+	{
+		if(m_RecvUnpacker.m_Data.m_DataSize >= 512)
+		{
+			SendTokenSixup(Addr, ResponseToken);
+			return 0;
+		}
+
+		// Is this behaviour safe to rely on?
+		pChunk->m_Flags = 0;
+		pChunk->m_ClientID = -1;
+		pChunk->m_Address = Addr;
+		pChunk->m_DataSize = 0;
+		return 1;
+	}
+	else if(ControlMsg == NET_CTRLMSG_CONNECT)
+	{
+		SECURITY_TOKEN MyToken = GetToken(Addr);
+		unsigned char aToken[4];
+		mem_copy(aToken, &MyToken, 4);
+
+		CNetBase::SendControlMsg(m_Socket, &Addr, 0, NET_CTRLMSG_CONNECTACCEPT, aToken, sizeof(aToken), ResponseToken, true);
+		if(Token == MyToken)
+			TryAcceptClient(Addr, ResponseToken, false, true, Token);
+	}
+
+	return 0;
+}
+
 int CNetServer::GetClientSlot(const NETADDR &Addr)
 {
 	int Slot = -1;
@@ -590,7 +633,7 @@ static bool IsDDNetControlMsg(const CNetPacketConstruct *pPacket)
 /*
 	TODO: chopp up this function into smaller working parts
 */
-int CNetServer::Recv(CNetChunk *pChunk)
+int CNetServer::Recv(CNetChunk *pChunk, SECURITY_TOKEN *ResponseToken)
 {
 	while(1)
 	{
@@ -617,10 +660,16 @@ int CNetServer::Recv(CNetChunk *pChunk)
 			continue;
 		}
 
-		if(CNetBase::UnpackPacket(pData, Bytes, &m_RecvUnpacker.m_Data) == 0)
+		SECURITY_TOKEN Token;
+		bool Sixup = false;
+		*ResponseToken = NET_SECURITY_TOKEN_UNKNOWN;
+		if(CNetBase::UnpackPacket(pData, Bytes, &m_RecvUnpacker.m_Data, Sixup, &Token, ResponseToken) == 0)
 		{
 			if(m_RecvUnpacker.m_Data.m_Flags&NET_PACKETFLAG_CONNLESS)
 			{
+				if(Sixup && Token != GetToken(Addr))
+					continue;
+
 				pChunk->m_Flags = NETSENDFLAG_CONNLESS;
 				pChunk->m_ClientID = -1;
 				pChunk->m_Address = Addr;
@@ -643,6 +692,13 @@ int CNetServer::Recv(CNetChunk *pChunk)
 				// normal packet, find matching slot
 				int Slot = GetClientSlot(Addr);
 
+				if(!Sixup && Slot != -1 && m_aSlots[Slot].m_Connection.m_Sixup)
+				{
+					Sixup = true;
+					if(CNetBase::UnpackPacket(pData, Bytes, &m_RecvUnpacker.m_Data, Sixup, &Token))
+						continue;
+				}
+
 				if (Slot != -1)
 				{
 					// found
@@ -651,7 +707,7 @@ int CNetServer::Recv(CNetChunk *pChunk)
 					if(m_RecvUnpacker.m_Data.m_Flags&NET_PACKETFLAG_CONTROL)
 						OnConnCtrlMsg(Addr, Slot, m_RecvUnpacker.m_Data.m_aChunkData[0], m_RecvUnpacker.m_Data);
 
-					if(m_aSlots[Slot].m_Connection.Feed(&m_RecvUnpacker.m_Data, &Addr))
+					if(m_aSlots[Slot].m_Connection.Feed(&m_RecvUnpacker.m_Data, &Addr, Token))
 					{
 						if(m_RecvUnpacker.m_Data.m_DataSize)
 							m_RecvUnpacker.Start(&Addr, &m_aSlots[Slot].m_Connection, Slot);
@@ -661,7 +717,13 @@ int CNetServer::Recv(CNetChunk *pChunk)
 				{
 					// not found, client that wants to connect
 
-					if(IsDDNetControlMsg(&m_RecvUnpacker.m_Data))
+					if(Sixup)
+					{
+						// got 0.7 control msg
+						if(OnSixupCtrlMsg(Addr, pChunk, m_RecvUnpacker.m_Data.m_aChunkData[0], m_RecvUnpacker.m_Data, *ResponseToken, Token) == 1)
+							return 1;
+					}
+					else if(IsDDNetControlMsg(&m_RecvUnpacker.m_Data))
 						// got ddnet control msg
 						OnTokenCtrlMsg(Addr, m_RecvUnpacker.m_Data.m_aChunkData[0], m_RecvUnpacker.m_Data);
 					else
@@ -710,6 +772,31 @@ int CNetServer::Send(CNetChunk *pChunk)
 	return 0;
 }
 
+void CNetServer::SendTokenSixup(NETADDR &Addr, SECURITY_TOKEN Token)
+{
+	SECURITY_TOKEN MyToken = GetToken(Addr);
+	unsigned char aBuf[512] = {};
+	mem_copy(aBuf, &MyToken, 4);
+	int Size = (Token == NET_SECURITY_TOKEN_UNKNOWN) ? 512 : 4;
+	CNetBase::SendControlMsg(m_Socket, &Addr, 0, 5, aBuf, Size, Token, true);
+}
+
+int CNetServer::SendConnlessSixup(CNetChunk *pChunk, SECURITY_TOKEN ResponseToken)
+{
+	if(pChunk->m_DataSize > NET_MAX_PACKETSIZE - 9)
+		return -1;
+
+	unsigned char aBuffer[NET_MAX_PACKETSIZE];
+	aBuffer[0] = NET_PACKETFLAG_CONNLESS<<2 | 1;
+	SECURITY_TOKEN Token = GetToken(pChunk->m_Address);
+	mem_copy(aBuffer+1, &ResponseToken, 4);
+	mem_copy(aBuffer+5, &Token, 4);
+	mem_copy(aBuffer+9, pChunk->m_pData, pChunk->m_DataSize);
+	net_udp_send(m_Socket, &pChunk->m_Address, aBuffer, pChunk->m_DataSize + 9);
+
+	return 0;
+}
+
 void CNetServer::SetMaxClientsPerIP(int Max)
 {
 	// clamp
@@ -726,7 +813,7 @@ bool CNetServer::SetTimedOut(int ClientID, int OrigID)
 	if (m_aSlots[ClientID].m_Connection.State() != NET_CONNSTATE_ERROR)
 		return false;
 
-	m_aSlots[ClientID].m_Connection.SetTimedOut(ClientAddr(OrigID), m_aSlots[OrigID].m_Connection.SeqSequence(), m_aSlots[OrigID].m_Connection.AckSequence(), m_aSlots[OrigID].m_Connection.SecurityToken(), m_aSlots[OrigID].m_Connection.ResendBuffer());
+	m_aSlots[ClientID].m_Connection.SetTimedOut(ClientAddr(OrigID), m_aSlots[OrigID].m_Connection.SeqSequence(), m_aSlots[OrigID].m_Connection.AckSequence(), m_aSlots[OrigID].m_Connection.SecurityToken(), m_aSlots[OrigID].m_Connection.ResendBuffer(), m_aSlots[OrigID].m_Connection.m_Sixup);
 	m_aSlots[OrigID].m_Connection.Reset();
 	return true;
 }
