@@ -78,8 +78,10 @@ enum
 enum
 {
 	SERVER_TICK_SPEED=50,
-	SERVER_FLAG_PASSWORD = 0x1,
-	SERVER_FLAG_ISDDNET = 0x2,
+	SERVER_FLAG_PASSWORD = 1<<0,
+	SERVER_FLAG_TIMESCORE = 1<<1,
+	SERVERINFO_LEVEL_MIN = 0,
+	SERVERINFO_LEVEL_MAX = 2,
 
 	MAX_CLIENTS=64,
 	VANILLA_MAX_CLIENTS=16,
@@ -100,6 +102,7 @@ enum
 
 enum
 {
+	VERSION_NONE = -1,
 	VERSION_VANILLA = 0,
 	VERSION_DDRACE = 1,
 	VERSION_DDNET_OLD = 2,
