@@ -8,24 +8,26 @@ class IVideo
 public:
 	virtual ~IVideo() {};
 
-	virtual void start() = 0;
-	virtual void stop() = 0;
+	virtual void Start() = 0;
+	virtual void Stop() = 0;
+	virtual void Pause(bool Pause) = 0;
+	virtual bool IsRecording() = 0;
 
-	virtual void nextVideoFrame() = 0;
-	virtual bool frameRendered() = 0;
-	virtual void nextVideoFrame_thread() = 0;
+	virtual void NextVideoFrame() = 0;
+	virtual bool FrameRendered() = 0;
+	virtual void NextVideoFrameThread() = 0;
 
-	virtual void nextAudioFrame(void (*Mix)(short *pFinalOut, unsigned Frames)) = 0;
-	virtual bool aframeRendered() = 0;
-	virtual void nextAudioFrame_timeline() = 0;
+	virtual void NextAudioFrame(void (*Mix)(short *pFinalOut, unsigned Frames)) = 0;
+	virtual bool AudioFrameRendered() = 0;
+	virtual void NextAudioFrameTimeline() = 0;
 
 
 	static IVideo* Current() { return ms_pCurrentVideo; }
 
-	static int64 time() { return ms_Time; }
+	static int64 Time() { return ms_Time; }
 	static float LocalTime() { return ms_LocalTime; }
 	static void SetLocalStartTime(int64 LocalStartTime) { ms_LocalStartTime = LocalStartTime; }
-	static void SetFPS(int fps) { ms_TickTime = time_freq() / fps; }
+	static void SetFPS(int FPS) { ms_TickTime = time_freq() / FPS; }
 
 protected:
 	static IVideo* ms_pCurrentVideo;
