@@ -4,6 +4,8 @@
 #include <base/math.h>
 #include <base/vmath.h>
 
+#include <antibot/antibot_data.h>
+
 #include <math.h>
 #include <engine/map.h>
 #include <engine/kernel.h>
@@ -152,6 +154,21 @@ void CCollision::Init(class CLayers *pLayers)
 	}
 }
 
+void CCollision::FillAntibot(CAntibotMapData *pMapData)
+{
+	pMapData->m_Width = m_Width;
+	pMapData->m_Height = m_Height;
+	pMapData->m_pTiles = (unsigned char *)malloc(m_Width * m_Height);
+	for(int i = 0; i < m_Width * m_Height; i++)
+	{
+		pMapData->m_pTiles[i] = 0;
+		if(m_pTiles[i].m_Index >= TILE_SOLID && m_pTiles[i].m_Index <= TILE_NOLASER)
+		{
+			pMapData->m_pTiles[i] = m_pTiles[i].m_Index;
+		}
+	}
+}
+
 enum
 {
 	MR_DIR_HERE=0,
