@@ -18,14 +18,14 @@ static struct
 {
 	CUIRect m_Rect;
 	void *m_pId;
-	int (*m_pfnFunc)(CEditor *pEditor, CUIRect Rect);
+	int (*m_pfnFunc)(CEditor *pEditor, CUIRect Rect, void *pContext);
 	int m_IsMenu;
-	void *m_pExtra;
+	void *m_pContext;
 } s_UiPopups[8];
 
 static int g_UiNumPopups = 0;
 
-void CEditor::UiInvokePopupMenu(void *pID, int Flags, float x, float y, float Width, float Height, int (*pfnFunc)(CEditor *pEditor, CUIRect Rect), void *pExtra)
+void CEditor::UiInvokePopupMenu(void *pID, int Flags, float x, float y, float Width, float Height, int (*pfnFunc)(CEditor *pEditor, CUIRect Rect, void *pContext), void *pContext)
 {
 	if(g_UiNumPopups > 7)
 		return;
@@ -41,7 +41,7 @@ void CEditor::UiInvokePopupMenu(void *pID, int Flags, float x, float y, float Wi
 	s_UiPopups[g_UiNumPopups].m_Rect.w = Width;
 	s_UiPopups[g_UiNumPopups].m_Rect.h = Height;
 	s_UiPopups[g_UiNumPopups].m_pfnFunc = pfnFunc;
-	s_UiPopups[g_UiNumPopups].m_pExtra = pExtra;
+	s_UiPopups[g_UiNumPopups].m_pContext = pContext;
 	g_UiNumPopups++;
 }
 
@@ -80,7 +80,7 @@ void CEditor::UiDoPopupMenu()
 		RenderTools()->DrawUIRect(&r, ColorRGBA(0,0,0,0.75f), Corners, 3.0f);
 		r.Margin(4.0f, &r);
 
-		if(s_UiPopups[i].m_pfnFunc(this, r))
+		if(s_UiPopups[i].m_pfnFunc(this, r, s_UiPopups[i].m_pContext))
 		{
 			m_LockMouse = false;
 			UI()->SetActiveItem(0);
@@ -98,8 +98,18 @@ void CEditor::UiDoPopupMenu()
 	}
 }
 
+bool CEditor::UiPopupExists(void *pid)
+{
+	for(int i = 0; i < g_UiNumPopups; i++)
+	{
+		if(s_UiPopups[i].m_pId == pid)
+			return true;
+	}
 
-int CEditor::PopupGroup(CEditor *pEditor, CUIRect View)
+	return false;
+}
+
+int CEditor::PopupGroup(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	// remove group button
 	CUIRect Button;
@@ -357,13 +367,20 @@ int CEditor::PopupGroup(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupLayer(CEditor *pEditor, CUIRect View)
+int CEditor::PopupLayer(CEditor *pEditor, CUIRect View, void *pContext)
 {
+	CLayerPopupContext *pPopup = (CLayerPopupContext *)pContext;
+
 	// remove layer button
 	CUIRect Button;
 	View.HSplitBottom(12.0f, &View, &Button);
 	static int s_DeleteButton = 0;
 
+	if(pPopup->m_aLayers.size() > 1)
+	{
+		return CLayerTiles::RenderCommonProperties(pPopup->m_CommonPropState, pEditor, &View, pPopup->m_aLayers);
+	}
+
 	// don't allow deletion of game layer
 	if(pEditor->m_Map.m_pGameLayer != pEditor->GetSelectedLayer(0) &&
 		pEditor->DoButton_Editor(&s_DeleteButton, "Delete layer", 0, &Button, 0, "Deletes the layer"))
@@ -452,7 +469,7 @@ int CEditor::PopupLayer(CEditor *pEditor, CUIRect View)
 	return pCurrentLayer->RenderProperties(&View);
 }
 
-int CEditor::PopupQuad(CEditor *pEditor, CUIRect View)
+int CEditor::PopupQuad(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	array<CQuad *> lQuads = pEditor->GetSelectedQuads();
 	CQuad *pCurrentQuad = lQuads[pEditor->m_SelectedQuadIndex];
@@ -634,7 +651,7 @@ int CEditor::PopupQuad(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupSource(CEditor *pEditor, CUIRect View)
+int CEditor::PopupSource(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CSoundSource *pSource = pEditor->GetSelectedSource();
 
@@ -828,7 +845,7 @@ int CEditor::PopupSource(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupPoint(CEditor *pEditor, CUIRect View)
+int CEditor::PopupPoint(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	array<CQuad *> lQuads = pEditor->GetSelectedQuads();
 	CQuad *pCurrentQuad = lQuads[pEditor->m_SelectedQuadIndex];
@@ -918,7 +935,7 @@ int CEditor::PopupPoint(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupNewFolder(CEditor *pEditor, CUIRect View)
+int CEditor::PopupNewFolder(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect Label, ButtonBar;
 
@@ -987,7 +1004,7 @@ int CEditor::PopupNewFolder(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupMapInfo(CEditor *pEditor, CUIRect View)
+int CEditor::PopupMapInfo(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect Label, ButtonBar, Button;
 
@@ -1055,7 +1072,7 @@ int CEditor::PopupMapInfo(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupEvent(CEditor *pEditor, CUIRect View)
+int CEditor::PopupEvent(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect Label, ButtonBar;
 
@@ -1074,6 +1091,8 @@ int CEditor::PopupEvent(CEditor *pEditor, CUIRect View)
 		pEditor->UI()->DoLabel(&Label, "Large layer", 20.0f, 0);
 	else if(pEditor->m_PopupEventType == POPEVENT_PREVENTUNUSEDTILES)
 		pEditor->UI()->DoLabel(&Label, "Unused tiles disabled", 20.0f, 0);
+	else if(pEditor->m_PopupEventType == POPEVENT_IMAGEDIV16)
+		pEditor->UI()->DoLabel(&Label, "Image width", 20.0f, 0);
 
 	View.HSplitBottom(10.0f, &View, 0);
 	View.HSplitBottom(20.0f, &View, &ButtonBar);
@@ -1095,6 +1114,8 @@ int CEditor::PopupEvent(CEditor *pEditor, CUIRect View)
 		pEditor->UI()->DoLabel(&Label, "You are trying to set the height or width of a layer to more than 1000 tiles. This is actually possible, but only rarely necessary. It may cause the editor to work slower, larger file size as well as higher memory usage for client and server.", 10.0f, -1, Label.w-10.0f);
 	else if(pEditor->m_PopupEventType == POPEVENT_PREVENTUNUSEDTILES)
 		pEditor->UI()->DoLabel(&Label, "Unused tiles can't be placed by default because they could get a use later and then destroy your map.\nActivate the 'Unused' switch to be able to place every tile.", 10.0f, -1, Label.w-10.0f);
+	else if(pEditor->m_PopupEventType == POPEVENT_IMAGEDIV16)
+		pEditor->UI()->DoLabel(&Label, "The width of this image is not divisible by 16. This is required for images used in tile layers for Teeworlds 0.7 compatibility.", 10.0f, -1, Label.w-10.0f);
 
 	// button bar
 	ButtonBar.VSplitLeft(30.0f, 0, &ButtonBar);
@@ -1120,7 +1141,7 @@ int CEditor::PopupEvent(CEditor *pEditor, CUIRect View)
 	}
 	ButtonBar.VSplitRight(30.0f, &ButtonBar, 0);
 	ButtonBar.VSplitRight(110.0f, &ButtonBar, &Label);
-	if(pEditor->m_PopupEventType != POPEVENT_LARGELAYER && pEditor->m_PopupEventType != POPEVENT_PREVENTUNUSEDTILES)
+	if(pEditor->m_PopupEventType != POPEVENT_LARGELAYER && pEditor->m_PopupEventType != POPEVENT_PREVENTUNUSEDTILES && pEditor->m_PopupEventType != POPEVENT_IMAGEDIV16)
 	{
 		static int s_AbortButton = 0;
 		if(pEditor->DoButton_Editor(&s_AbortButton, "Abort", 0, &Label, 0, 0))
@@ -1137,7 +1158,7 @@ int CEditor::PopupEvent(CEditor *pEditor, CUIRect View)
 static int g_SelectImageSelected = -100;
 static int g_SelectImageCurrent = -100;
 
-int CEditor::PopupSelectImage(CEditor *pEditor, CUIRect View)
+int CEditor::PopupSelectImage(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect ButtonBar, ImageView;
 	View.VSplitLeft(80.0f, &ButtonBar, &View);
@@ -1249,7 +1270,7 @@ int CEditor::PopupSelectImageResult()
 static int g_SelectSoundSelected = -100;
 static int g_SelectSoundCurrent = -100;
 
-int CEditor::PopupSelectSound(CEditor *pEditor, CUIRect View)
+int CEditor::PopupSelectSound(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect ButtonBar, SoundView;
 	View.VSplitLeft(80.0f, &ButtonBar, &View);
@@ -1340,7 +1361,7 @@ int CEditor::PopupSelectSoundResult()
 
 static int s_GametileOpSelected = -1;
 
-int CEditor::PopupSelectGametileOp(CEditor *pEditor, CUIRect View)
+int CEditor::PopupSelectGametileOp(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	static const char *s_pButtonNames[] = { "Air", "Hookable", "Death", "Unhookable", "Hookthrough", "Freeze", "Unfreeze", "Deep Freeze", "Deep Unfreeze", "Blue Check-Tele", "Red Check-Tele" };
 	static unsigned s_NumButtons = sizeof(s_pButtonNames) / sizeof(char*);
@@ -1377,7 +1398,7 @@ int CEditor::PopupSelectGameTileOpResult()
 static int s_AutoMapConfigSelected = -100;
 static int s_AutoMapConfigCurrent = -100;
 
-int CEditor::PopupSelectConfigAutoMap(CEditor *pEditor, CUIRect View)
+int CEditor::PopupSelectConfigAutoMap(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CLayerTiles *pLayer = static_cast<CLayerTiles*>(pEditor->GetSelectedLayer(0));
 	CUIRect Button;
@@ -1416,55 +1437,76 @@ int CEditor::PopupSelectConfigAutoMapResult()
 
 // DDRace
 
-int CEditor::PopupTele(CEditor *pEditor, CUIRect View)
+int CEditor::PopupTele(CEditor *pEditor, CUIRect View, void *pContext)
 {
-	CUIRect Button;
-	View.HSplitBottom(12.0f, &View, &Button);
+	static int s_PreviousNumber = -1;
 
-	enum
-	{
-		PROP_TELE=0,
-		NUM_PROPS,
-	};
+	CUIRect NumberPicker;
+	CUIRect FindEmptySlot;
 
-	CProperty aProps[] = {
-		{"Number", pEditor->m_TeleNumber, PROPTYPE_INT_STEP, 0, 255},
-		{0},
-	};
+	View.VSplitRight(15.f, &NumberPicker, &FindEmptySlot);
+	NumberPicker.VSplitRight(2.f, &NumberPicker, 0);
 
-	static int s_aIds[NUM_PROPS] = {0};
-	int NewVal = 0;
-	static ColorRGBA s_color = ColorRGBA(1,1,1,0.5f);
-
-	int Prop = pEditor->DoProperties(&View, aProps, s_aIds, &NewVal, s_color);
-
-	if(Prop == PROP_TELE)
-	{
-		NewVal = (NewVal + 256) % 256;
-
-		CLayerTele *gl = pEditor->m_Map.m_pTeleLayer;
-		for(int y = 0; y < gl->m_Height; ++y)
+	// find empty number button
+	{ 
+		static int s_EmptySlotPid = 0;
+		if(pEditor->DoButton_Editor(&s_EmptySlotPid, "F", 0, &FindEmptySlot, 0, "[ctrl+f] Find empty slot")
+			|| pEditor->Input()->KeyPress(KEY_F))
 		{
-			for(int x = 0; x < gl->m_Width; ++x)
+			int number = -1;
+			for(int i = 1; i <= 255; i++)
 			{
-				if(gl->m_pTeleTile[y*gl->m_Width+x].m_Number == NewVal)
+				if(!pEditor->m_Map.m_pTeleLayer->ContainsElementWithId(i))
 				{
-					s_color = ColorRGBA(1,0.5f,0.5f,0.5f);
-					goto done;
+					number = i;
+					break;
 				}
 			}
+
+			if(number != -1)
+			{
+				pEditor->m_TeleNumber = number;
+			}
+		}
+	}
+
+	// number picker
+	{
+		static ColorRGBA s_Color = ColorRGBA(0.5f, 1, 0.5f, 0.5f);
+
+		enum
+		{
+			PROP_TELE = 0,
+			NUM_PROPS,
+		};
+		CProperty aProps[] = {
+			{"Number", pEditor->m_TeleNumber, PROPTYPE_INT_STEP, 1, 255},
+			{0},
+		};
+		
+		static int s_aIds[NUM_PROPS] = {0};
+
+		static int NewVal = 0;
+		int Prop = pEditor->DoProperties(&NumberPicker, aProps, s_aIds, &NewVal, s_Color);
+		if(Prop == PROP_TELE)
+		{
+			pEditor->m_TeleNumber = (NewVal + 256) % 256;
 		}
 
-		s_color = ColorRGBA(0.5f,1,0.5f,0.5f);
-
-		done:
-		pEditor->m_TeleNumber = NewVal;
+		if(s_PreviousNumber == 1 || s_PreviousNumber != pEditor->m_TeleNumber)
+		{
+			s_Color = pEditor->m_Map.m_pTeleLayer->ContainsElementWithId(pEditor->m_TeleNumber)
+				? ColorRGBA(1, 0.5f, 0.5f, 0.5f)
+				: ColorRGBA(0.5f, 1, 0.5f, 0.5f);
+		}
 	}
 
+	s_PreviousNumber = pEditor->m_TeleNumber;
+
 	return 0;
 }
 
-int CEditor::PopupSpeedup(CEditor *pEditor, CUIRect View)
+int CEditor::PopupSpeedup(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect Button;
 	View.HSplitBottom(12.0f, &View, &Button);
@@ -1498,58 +1540,85 @@ int CEditor::PopupSpeedup(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupSwitch(CEditor *pEditor, CUIRect View)
+int CEditor::PopupSwitch(CEditor *pEditor, CUIRect View, void *pContext)
 {
-	CUIRect Button;
-	View.HSplitBottom(12.0f, &View, &Button);
+	static int s_PreviousNumber = -1;
 
-	enum
+	CUIRect NumberPicker;
+	CUIRect FindEmptySlot;
+
+	CUIRect DelayPicker;
+
+	View.HSplitMid(&NumberPicker, &DelayPicker);
+	NumberPicker.VSplitRight(15.f, &NumberPicker, &FindEmptySlot);
+	NumberPicker.VSplitRight(2.f, &NumberPicker, 0);
+
+	// find empty number button
 	{
-		PROP_SwitchDelay=0,
-		PROP_SwitchNumber,
-		NUM_PROPS,
-	};
-
-	CProperty aProps[] = {
-		{"Delay", pEditor->m_SwitchDelay, PROPTYPE_INT_STEP, 0, 255},
-		{"Number", pEditor->m_SwitchNum, PROPTYPE_INT_STEP, 0, 255},
-		{0},
-	};
-
-	static int s_aIds[NUM_PROPS] = {0};
-	int NewVal = 0;
-	static ColorRGBA s_color = ColorRGBA(1,1,1,0.5f);
-	int Prop = pEditor->DoProperties(&View, aProps, s_aIds, &NewVal, s_color);
-
-	if(Prop == PROP_SwitchNumber)
-	{
-		NewVal = (NewVal + 256) % 256;
-
-		CLayerSwitch *gl = pEditor->m_Map.m_pSwitchLayer;
-		for(int y = 0; y < gl->m_Height; ++y)
+		static int s_EmptySlotPid = 0;
+		if(pEditor->DoButton_Editor(&s_EmptySlotPid, "F", 0, &FindEmptySlot, 0, "[ctrl+f] Find empty slot")
+			|| pEditor->Input()->KeyPress(KEY_F))
 		{
-			for(int x = 0; x < gl->m_Width; ++x)
+			int number = -1;
+			for(int i = 1; i <= 255; i++)
 			{
-				if(gl->m_pSwitchTile[y*gl->m_Width+x].m_Number == NewVal)
+				if(!pEditor->m_Map.m_pSwitchLayer->ContainsElementWithId(i))
 				{
-					s_color = ColorRGBA(1,0.5f,0.5f,0.5f);
-					goto done;
+					number = i;
+					break;
 				}
 			}
+
+			if(number != -1)
+			{
+				pEditor->m_SwitchNum = number;
+			}
+		}
+	}
+
+	// number picker
+	{
+		static ColorRGBA s_Color = ColorRGBA(1, 1, 1, 0.5f);
+
+		enum
+		{
+			PROP_SwitchNumber = 0,
+			PROP_SwitchDelay,
+			NUM_PROPS,
+		};
+
+		CProperty aProps[] = {
+			{"Number", pEditor->m_SwitchNum, PROPTYPE_INT_STEP, 1, 255},
+			{"Delay", pEditor->m_SwitchDelay, PROPTYPE_INT_STEP, 0, 255},
+			{0},
+		}; 
+
+		static int s_aIds[NUM_PROPS] = {0};
+		int NewVal = 0;
+		int Prop = pEditor->DoProperties(&NumberPicker, aProps, s_aIds, &NewVal, s_Color);
+
+		if(Prop == PROP_SwitchNumber)
+		{
+			pEditor->m_SwitchNum = (NewVal + 256) % 256;
+		}
+		else if(Prop == PROP_SwitchDelay)
+		{
+			pEditor->m_SwitchDelay = (NewVal + 256) % 256;
 		}
 
-		s_color = ColorRGBA(0.5f,1,0.5f,0.5f);
-
-		done:
-		pEditor->m_SwitchNum = NewVal;
+		if(s_PreviousNumber == 1 || s_PreviousNumber != pEditor->m_SwitchNum)
+		{
+			s_Color = pEditor->m_Map.m_pSwitchLayer->ContainsElementWithId(pEditor->m_SwitchNum)
+				? ColorRGBA(1, 0.5f, 0.5f, 0.5f)
+				: ColorRGBA(0.5f, 1, 0.5f, 0.5f);
+		}
 	}
-	if(Prop == PROP_SwitchDelay)
-		pEditor->m_SwitchDelay = (NewVal + 256) % 256;
 
+	s_PreviousNumber = pEditor->m_SwitchNum;
 	return 0;
 }
 
-int CEditor::PopupTune(CEditor *pEditor, CUIRect View)
+int CEditor::PopupTune(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect Button;
 	View.HSplitBottom(12.0f, &View, &Button);
@@ -1575,7 +1644,7 @@ int CEditor::PopupTune(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupColorPicker(CEditor *pEditor, CUIRect View)
+int CEditor::PopupColorPicker(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	CUIRect SVPicker, HuePicker;
 	View.VSplitRight(20.0f, &SVPicker, &HuePicker);
@@ -1677,7 +1746,7 @@ int CEditor::PopupColorPicker(CEditor *pEditor, CUIRect View)
 	return 0;
 }
 
-int CEditor::PopupEntities(CEditor *pEditor, CUIRect View)
+int CEditor::PopupEntities(CEditor *pEditor, CUIRect View, void *pContext)
 {
 	for(int i = 0; i < (int)pEditor->m_SelectEntitiesFiles.size(); i++)
 	{
@@ -1697,6 +1766,7 @@ int CEditor::PopupEntities(CEditor *pEditor, CUIRect View)
 
 				pEditor->Graphics()->UnloadTexture(pEditor->m_EntitiesTexture);
 				pEditor->m_EntitiesTexture = pEditor->Graphics()->LoadTexture(aBuf, IStorage::TYPE_ALL, CImageInfo::FORMAT_AUTO, 0);
+				g_UiNumPopups--;
 			}
 		}
 	}
