@@ -14,6 +14,7 @@
 #include <engine/shared/protocol.h>
 #include <game/generated/protocol.h>
 
+#include "prng.h"
 #include "teamscore.h"
 #include "mapitems.h"
 
@@ -112,8 +113,15 @@ inline void IntsToStr(const int *pInts, int Num, char *pStr)
 		Num--;
 	}
 
+#if defined(__GNUC__) && __GNUC__ >= 7
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-overflow" // false positive
+#endif
 	// null terminate
 	pStr[-1] = 0;
+#if defined(__GNUC__) && __GNUC__ >= 7
+#pragma GCC diagnostic pop
+#endif
 }
 
 
@@ -180,10 +188,24 @@ public:
 	CWorldCore()
 	{
 		mem_zero(m_apCharacters, sizeof(m_apCharacters));
+		m_pPrng = 0;
+	}
+
+	int RandomOr0(int BelowThis)
+	{
+		if(BelowThis <= 1 || !m_pPrng)
+		{
+			return 0;
+		}
+		// This makes the random number slightly biased if `BelowThis`
+		// is not a power of two, but we have decided that this is not
+		// significant for DDNet and favored the simple implementation.
+		return m_pPrng->RandomBits() % BelowThis;
 	}
 
 	CTuningParams m_Tuning[2];
 	class CCharacterCore *m_apCharacters[MAX_CLIENTS];
+	CPrng *m_pPrng;
 };
 
 class CCharacterCore
@@ -257,11 +279,6 @@ public:
 	int m_FreezeEnd;
 	bool m_DeepFrozen;
 
-	// Caps the given velocity according to the current set of stoppers
-	// that the character is affected by.
-	vec2 LimitVel(vec2 Vel);
-	void ApplyForce(vec2 Force);
-
 private:
 
 	CTeamsCore *m_pTeams;
