@@ -2,9 +2,10 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #ifndef GAME_VERSION_H
 #define GAME_VERSION_H
-#define GAME_VERSION "0.6.4, 12.9.2"
+#define GAME_VERSION "0.6.4, 14.1"
 #define GAME_NETVERSION "0.6 626fce9a778df4d4"
-#define GAME_RELEASE_VERSION "12.9.2"
-#define CLIENT_VERSIONNR 12092
+#define GAME_NAME "DDNet"
+#define GAME_RELEASE_VERSION "14.1"
+#define CLIENT_VERSIONNR 14010
 extern const char *GIT_SHORTREV_HASH;
 #endif
